const ObjectDiffTracker = require('object-diff-tracker');

class StateManagementSystem {
    constructor(initialState = {}) {
        this.diffTracker = new ObjectDiffTracker();
        this.stateHistory = [];
        this.currentStateIndex = -1;

        this.setState(initialState);
    }

    setState(newState) {
        this.diffTracker.setObject(newState);
        this.recordState();
    }

    updateState(partialState) {
        // Merge partial state into current state
        Object.assign(this.diffTracker.currentObject, partialState);
        this.diffTracker.trackChanges();
        this.recordState();
    }

    recordState() {
        const currentState = JSON.parse(JSON.stringify(this.diffTracker.currentObject));
        // Trim future states if we're in the middle of the history
        this.stateHistory = this.stateHistory.slice(0, this.currentStateIndex + 1);
        this.stateHistory.push(currentState);
        this.currentStateIndex++;
    }

    undo() {
        if (this.currentStateIndex > 0) {
            this.currentStateIndex--;
            this.applyState(this.currentStateIndex);
        }
    }

    redo() {
        if (this.currentStateIndex < this.stateHistory.length - 1) {
            this.currentStateIndex++;
            this.applyState(this.currentStateIndex);
        }
    }

    applyState(index) {
        const stateToApply = this.stateHistory[index];
        this.diffTracker.setObject(stateToApply);
    }

    getCurrentState() {
        return this.diffTracker.currentObject;
    }

    getChanges() {
        return this.diffTracker.getChanges();
    }
}

module.exports = StateManagementSystem;